@@ -15,6 +15,7 @@
 
 import abc
 import copy
+import functools
 import inspect
 import threading
 
@@ -52,6 +53,34 @@ INCONSISTENCY_TYPE_CREATE_UPDATE = 'create/update'
 INCONSISTENCY_TYPE_DELETE = 'delete'
 
 
+def has_lock_periodic(*args, periodic_run_limit=0, **kwargs):
+    def wrapper(f):
+        _retries = 0
+
+        @functools.wraps(f)
+        @periodics.periodic(*args, **kwargs)
+        def decorator(self, *args, **kwargs):
+            # This periodic task is included in DBInconsistenciesPeriodics
+            # since it uses the lock to ensure only one worker is executing
+            # additonally, if periodic_run_limit parameter with value > 0 is
+            # provided and lock is not acquired for periodic_run_limit
+            # times, task will not be run anymore by this maintenance worker
+            nonlocal _retries
+            if not self.has_lock:
+                if periodic_run_limit > 0:
+                    if _retries >= periodic_run_limit:
+                        LOG.debug("Have not been able to acquire lock to run "
+                                  "task '%s' after %s tries, limit reached. "
+                                  "No more attempts will be made.",
+                                  f, _retries)
+                        raise periodics.NeverAgain()
+                    _retries += 1
+                return
+            return f(self, *args, **kwargs)
+        return decorator
+    return wrapper
+
+
 class MaintenanceThread(object):
 
     def __init__(self):
@@ -288,15 +317,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # is held by some other neutron-server instance in the cloud, we'll attempt
     # to perform the migration every 10 seconds until completed.
     # TODO(ihrachys): Remove the migration to stateful fips in Z+1.
-    @periodics.periodic(spacing=10, run_immediately=True)
+    @has_lock_periodic(spacing=10, run_immediately=True)
     @rerun_on_schema_updates
     def migrate_to_stateful_fips(self):
         """Perform the migration from stateless to stateful Floating IPs. """
-        # Only the worker holding a valid lock within OVSDB will perform the
-        # migration.
-        if not self.has_lock:
-            return
-
         admin_context = n_context.get_admin_context()
         nb_sync = ovn_db_sync.OvnNbSynchronizer(
             self._ovn_client._plugin, self._nb_idl, self._ovn_client._sb_idl,
@@ -357,14 +381,9 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         _log(create_update_inconsistencies, INCONSISTENCY_TYPE_CREATE_UPDATE)
         _log(delete_inconsistencies, INCONSISTENCY_TYPE_DELETE)
 
-    @periodics.periodic(spacing=ovn_const.DB_CONSISTENCY_CHECK_INTERVAL,
-                        run_immediately=True)
+    @has_lock_periodic(spacing=ovn_const.DB_CONSISTENCY_CHECK_INTERVAL,
+                       run_immediately=True)
     def check_for_inconsistencies(self):
-        # Only the worker holding a valid lock within OVSDB will run
-        # this periodic
-        if not self.has_lock:
-            return
-
         admin_context = n_context.get_admin_context()
         create_update_inconsistencies = (
             revision_numbers_db.get_inconsistent_resources(admin_context))
@@ -480,13 +499,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600,
-                        run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_global_dhcp_opts(self):
-        # This periodic task is included in DBInconsistenciesPeriodics since
-        # it uses the lock to ensure only one worker is executing
-        if not self.has_lock:
-            return
         if (not ovn_conf.get_global_dhcpv4_opts() and
                 not ovn_conf.get_global_dhcpv6_opts()):
             # No need to scan the subnets if the settings are unset.
@@ -515,11 +532,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_for_igmp_snoop_support(self):
-        if not self.has_lock:
-            return
-
         with self._nb_idl.transaction(check_error=True) as txn:
             value = ('true' if ovn_conf.is_igmp_snooping_enabled()
                      else 'false')
@@ -538,11 +555,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # TODO(czesla): Remove this in the A+4 cycle
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_port_has_address_scope(self):
-        if not self.has_lock:
-            return
-
         ports = self._nb_idl.db_find_rows(
             "Logical_Switch_Port", ("type", "!=", ovn_const.LSP_TYPE_LOCALNET)
         ).execute(check_error=True)
@@ -584,16 +601,16 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_for_ha_chassis_group(self):
         # If external ports is not supported stop running
         # this periodic task
         if not self._ovn_client.is_external_ports_supported():
             raise periodics.NeverAgain()
 
-        if not self.has_lock:
-            return
-
         external_ports = self._nb_idl.db_find_rows(
             'Logical_Switch_Port', ('type', '=', ovn_const.LSP_TYPE_EXTERNAL)
         ).execute(check_error=True)
@@ -617,11 +634,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # TODO(lucasagomes): Remove this in the B+3 cycle
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_for_mcast_flood_reports(self):
-        if not self.has_lock:
-            return
-
         cmds = []
         for port in self._nb_idl.lsp_list().execute(check_error=True):
             port_type = port.type.strip()
@@ -665,11 +682,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_localnet_port_has_learn_fdb(self):
-        if not self.has_lock:
-            return
-
         ports = self._nb_idl.db_find_rows(
             "Logical_Switch_Port", ("type", "=", ovn_const.LSP_TYPE_LOCALNET)
         ).execute(check_error=True)
@@ -696,11 +713,8 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # TODO(lucasagomes): Remove this in the Z cycle
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(spacing=600, run_immediately=True)
     def check_router_mac_binding_options(self):
-        if not self.has_lock:
-            return
-
         cmds = []
         for router in self._nb_idl.lr_list().execute(check_error=True):
             if (router.options.get('always_learn_from_arp_request') and
@@ -720,14 +734,15 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_redirect_type_router_gateway_ports(self):
         """Check OVN router gateway ports
         Check for the option "redirect-type=bridged" value for
         router gateway ports.
         """
-        if not self.has_lock:
-            return
         context = n_context.get_admin_context()
         cmds = []
         gw_ports = self._ovn_client._plugin.get_ports(
@@ -780,23 +795,21 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_vlan_distributed_ports(self):
         """Check VLAN distributed ports
         Check for the option "reside-on-redirect-chassis" value for
         distributed VLAN ports.
         """
-        if not self.has_lock:
-            return
         context = n_context.get_admin_context()
         cmds = []
         # Get router ports belonging to VLAN networks
         vlan_nets = self._ovn_client._plugin.get_networks(
             context, {pnet.NETWORK_TYPE: [n_const.TYPE_VLAN]})
-        # FIXME(ltomasbo): Once Bugzilla 2162756 is fixed the
-        # is_provider_network check should be removed
-        vlan_net_ids = [vn['id'] for vn in vlan_nets
-                        if not utils.is_provider_network(vn)]
+        vlan_net_ids = [vn['id'] for vn in vlan_nets]
         router_ports = self._ovn_client._plugin.get_ports(
             context, {'network_id': vlan_net_ids,
                       'device_owner': n_const.ROUTER_PORT_OWNERS})
@@ -823,12 +836,12 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # "external_ids:OVN_GW_PORT_EXT_ID_KEY" from to each router.
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def remove_gw_ext_ids_from_logical_router(self):
         """Remove `gw_port_id` and `gw_network_id` external_ids from LRs"""
-        if not self.has_lock:
-            return
-
         cmds = []
         for lr in self._nb_idl.lr_list().execute(check_error=True):
             if (ovn_const.OVN_GW_PORT_EXT_ID_KEY not in lr.external_ids and
@@ -854,7 +867,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_baremetal_ports_dhcp_options(self):
         """Update baremetal ports DHCP options
 
@@ -866,9 +882,6 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         if not self._ovn_client.is_external_ports_supported():
             raise periodics.NeverAgain()
 
-        if not self.has_lock:
-            return
-
         context = n_context.get_admin_context()
         ports = ports_obj.Port.get_ports_by_vnic_type_and_host(
             context, portbindings.VNIC_BAREMETAL)
@@ -906,16 +919,13 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         raise periodics.NeverAgain()
 
     # TODO(ralonsoh): Remove this in the Z+4 cycle
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(spacing=600, run_immediately=True)
     def update_port_virtual_type(self):
         """Set type=virtual to those ports with parents
         Before LP#1973276, any virtual port with "device_owner" defined, lost
         its type=virtual. This task restores the type for those ports updated
         before the fix https://review.opendev.org/c/openstack/neutron/+/841711.
         """
-        if not self.has_lock:
-            return
-
         context = n_context.get_admin_context()
         cmds = []
         for lsp in self._nb_idl.lsp_list().execute(check_error=True):
@@ -943,7 +953,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         raise periodics.NeverAgain()
 
     # TODO(ralonsoh): Remove this in the Antelope+4 cycle
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def create_router_extra_attributes_registers(self):
         """Create missing ``RouterExtraAttributes`` registers.
 
@@ -953,9 +966,6 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         only execution method finds those ``Routers`` registers without the
         child one and creates one with the default values.
         """
-        if not self.has_lock:
-            return
-
         context = n_context.get_admin_context()
         for router_id in router_obj.Router.\
                 get_router_ids_without_router_std_attrs(context):
@@ -967,7 +977,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         raise periodics.NeverAgain()
 
     # TODO(slaweq): Remove this in the E cycle (C+2 as it will be next SLURP)
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def add_gw_port_info_to_logical_router_port(self):
         """Add info if LRP is connecting internal subnet or ext gateway."""
         cmds = []
@@ -1003,13 +1016,13 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                     txn.add(cmd)
         raise periodics.NeverAgain()
 
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_router_default_route_empty_dst_ip(self):
         """Check routers with default route with empty dst-ip (LP: #2002993).
         """
-        if not self.has_lock:
-            return
-
         cmds = []
         for router in self._nb_idl.lr_list().execute(check_error=True):
             if not router.external_ids.get(ovn_const.OVN_REV_NUM_EXT_ID_KEY):
@@ -1031,7 +1044,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         raise periodics.NeverAgain()
 
     # TODO(ralonsoh): Remove this in the Antelope+4 cycle
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def add_vnic_type_and_pb_capabilities_to_lsp(self):
         """Add the port VNIC type and port binding capabilities to the LSP.
 
@@ -1042,9 +1058,6 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         been added to the LSP the VNIC type and the port binding capabilities.
         To implement LP#1998608, only direct ports are needed.
         """
-        if not self.has_lock:
-            return
-
         port_bindings = ports_obj.PortBinding.get_port_binding_by_vnic_type(
             n_context.get_admin_context(), portbindings.VNIC_DIRECT)
         with self._nb_idl.transaction(check_error=True) as txn:
@@ -1066,7 +1079,10 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
         raise periodics.NeverAgain()
 
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def check_fair_meter_consistency(self):
         """Update the logging meter after neutron-server reload
 
@@ -1075,8 +1091,6 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         driver after the OVN NB idl is loaded
 
         """
-        if not self.has_lock:
-            return
         if log_driver.OVNDriver.network_logging_supported(self._nb_idl):
             meter_name = (
                 cfg.CONF.network_log.local_output_log_base or "acl_log_meter")
@@ -1141,7 +1155,7 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                     for table in ('Chassis_Private', 'Chassis'):
                         txn.add(self._sb_idl.db_destroy(table, ch.name))
 
-    @periodics.periodic(spacing=86400, run_immediately=True)
+    @has_lock_periodic(spacing=86400, run_immediately=True)
     def cleanup_old_hash_ring_nodes(self):
         """Daily task to cleanup old stable Hash Ring node entries.
 
@@ -1150,12 +1164,13 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         information.
 
         """
-        if not self.has_lock:
-            return
         context = n_context.get_admin_context()
         hash_ring_db.cleanup_old_nodes(context, days=5)
 
-    @periodics.periodic(spacing=600, run_immediately=True)
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
     def update_nat_floating_ip_with_gateway_port_reference(self):
         """Set NAT rule gateway_port column to any floating IP without
         router gateway port uuid reference - LP#2035281.
@@ -1201,6 +1216,21 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                     txn.add(cmd)
         raise periodics.NeverAgain()
 
+    @has_lock_periodic(
+        periodic_run_limit=ovn_const.MAINTENANCE_TASK_RETRY_LIMIT,
+        spacing=ovn_const.MAINTENANCE_ONE_RUN_TASK_SPACING,
+        run_immediately=True)
+    def set_fip_distributed_flag(self):
+        """Set the NB_Global.external_ids:fip-distributed flag."""
+        distributed = ovn_conf.is_ovn_distributed_floating_ip()
+        LOG.debug(
+            "Setting fip-distributed flag in NB_Global to %s", distributed)
+        self._nb_idl.db_set(
+            'NB_Global', '.', external_ids={
+                ovn_const.OVN_FIP_DISTRIBUTED_KEY: str(distributed)}).execute(
+                    check_error=True)
+        raise periodics.NeverAgain()
+
 
 class HashRingHealthCheckPeriodics(object):
 
